<?php

namespace WPDesk\ShopMagic\AutomationOutcome;

use WPDesk\ShopMagic\Action\Action;
use WPDesk\ShopMagic\Automation\Automation;
use WPDesk\ShopMagic\Database\DatabaseSchema;
use WPDesk\ShopMagic\Database\RepositoryOrderTrait;
use WPDesk\ShopMagic\Event\Event;
use WPDesk\ShopMagic\Placeholder\Builtin\Customer\CustomerEmail;
use WPDesk\ShopMagic\Placeholder\Builtin\Customer\CustomerId;

/**
 * Kind of repositoruy pattern with some methods to insert outcomes.
 *
 * @package WPDesk\ShopMagic\AutomationOutcome
 */
final class OutcomeReposistory {
	use RepositoryOrderTrait;

	/**
	 * @param \wpdb|null $wpdb
	 */
	private $wpdb;

	public function __construct( \wpdb $wpdb ) {
		$this->wpdb = $wpdb;
	}

	/**
	 * @param string|int $execution_id
	 * @param string $note
	 */
	public function log_note( $execution_id, $note ) {
		$now = gmdate( 'Y-m-d G:i:s' );
		$this->insert_note( $execution_id, $note, $now );

		$outcome_table = DatabaseSchema::get_automation_outcome_table_name();
		$this->wpdb->insert( $outcome_table, [
			'updated' => $now,
		], [ 'execution_id' => $execution_id ] );
	}

	/**
	 * @param string|int $execution_id
	 * @param string $note
	 * @param int $now Time in unix UTC
	 */
	private function insert_note( $execution_id, $note, $now ) {
		$log_table = DatabaseSchema::get_outcome_logs_table_name();
		$this->wpdb->insert( $log_table, [
			'execution_id' => $execution_id,
			'note'         => $note,
			'created'      => $now,
		], [ 'execution_id' => $execution_id ] );
	}

	/**
	 * Run to set outcome as finished.
	 *
	 * @param string|int $execution_id
	 * @param bool $success
	 * @param string|null $finish_note
	 */
	public function finish_outcome( $execution_id, $success, $finish_note = null ) {
		$now = gmdate( 'Y-m-d G:i:s' );
		if ( $finish_note !== null ) {
			$this->insert_note( $execution_id, $finish_note, $now );
		}

		$outcome_table = DatabaseSchema::get_automation_outcome_table_name();
		$this->wpdb->update( $outcome_table, [
			'finished' => true,
			'success'  => $success,
			'updated'  => $now,
		], [ 'id' => $execution_id ] );
	}

	/**
	 * @param array $provided_data
	 *
	 * @return bool
	 */
	private function is_guest( $provided_data ) {
		if ( isset( $provided_data[ \WP_User::class ] ) ) {
			$user = $provided_data[ \WP_User::class ];

			return ! ( $user instanceof \WP_User && isset( $user->ID ) && $user->ID !== 0 );
		}

		return true;
	}

	/**
	 * Run to initialize outcome. Every outcome have to be initialized first.
	 *
	 * @param Automation $automation
	 * @param Event $event
	 * @param Action $action
	 * @param int $action_index
	 *
	 * @return string|int Unique outcome id to log future notices and savr finish.
	 */
	public function prepare_for_outcome( Automation $automation, Event $event, Action $action, $action_index ) {
		$unique_id = uniqid( 'execute_', true );

		// TODO: refactor when customer layer is completed. Now use placeholders and filters to check customer/guest/email info.
		$provided_data  = $event->get_provided_data();
		$customer_email = new CustomerEmail();
		$customer_email->set_provided_data( $provided_data );

		if ( $this->is_guest( $provided_data ) ) {
			$customer_id = null;
			$guest_id    = null;
		} else {
			$customer_id = new CustomerId();
			$customer_id->set_provided_data( $provided_data );
			$customer_id = $customer_id->value( [] );
			$guest_id    = null;
		}

		$outcome_table = DatabaseSchema::get_automation_outcome_table_name();
		$this->wpdb->insert( $outcome_table, [
			'execution_id'    => $unique_id,
			'automation_id'   => $automation->get_id(),
			'automation_name' => $automation->get_name(),
			'action_index'    => $action_index,
			'action_name'     => $action->get_name(),
			'customer_id'     => $customer_id,
			'guest_id'        => $guest_id,
			'customer_email'  => $customer_email->value( [] ),
			'success'         => null,
			'finished'        => false,
			'created'         => gmdate( 'Y-m-d G:i:s' ),
			'updated'         => gmdate( 'Y-m-d G:i:s' )
		] );

		return $this->wpdb->insert_id;
	}

	/**
	 * @return int
	 */
	public function get_count( array $where ) {
		$where      = array_merge( $where, [ 'finished' => true ] );
		$where_sql  = $this->where_array_to_sql( $where );
		$table_name = DatabaseSchema::get_automation_outcome_table_name();
		$sql        = "SELECT COUNT(*) FROM {$table_name} WHERE {$where_sql}";

		return (int) $this->wpdb->get_var( $sql );
	}

	/**
	 * @param array $where
	 * @param array $order
	 * @param int $limit
	 * @param int $offset
	 *
	 * @return \Generator|OutcomeInTable[]
	 * @throws \Exception
	 */
	public function get_all( array $where, array $order, $limit = 10, $offset = 0 ) {
		$table_name = DatabaseSchema::get_automation_outcome_table_name();
		$where      = array_merge( $where, [ 'finished' => true ] );
		$where_sql  = $this->where_array_to_sql( $where );
		$order_sql  = $this->order_array_to_sql( $order );
		$limit_sql  = $this->limit_offset_to_sql( $limit, $offset );

		$sql = "
			SELECT
       			id, automation_id, automation_name, customer_id, customer_email, action_name, action_index, updated, success
       		FROM
       		     {$table_name}
            WHERE
                  $where_sql
            {$order_sql} {$limit_sql}";

		foreach ( $this->wpdb->get_results( $sql, ARRAY_A ) as $item ) {
			yield new OutcomeInTable(
				$item['id'],
				$item['automation_id'],
				$item['automation_name'],
				$item['customer_id'],
				$item['customer_email'],
				empty( $item['action_name'] ) ? '#' . ( $item['action_index'] + 1 ) : $item['action_name'],
				$item['success'],
				new \DateTimeImmutable( $item['updated'] )
			);
		}
	}
}

