<?php
/**
 * Creates tables and populate data for plugin
 */

namespace WPDesk\ShopMagic\Database;

use WPDesk\ShopMagic\Frontend\ListsOnAccount;

class DatabaseSchema {
	const VERSION = 39;
	const OPTION_NAME_WITH_VERSION = 'shopmagic_db_version';

	/** @var string */
	private $base_file_name;

	/**
	 * @return string
	 */
	public static function get_optin_email_table_name() {
		global $wpdb;

		return $wpdb->prefix . 'shopmagic_optin_email';
	}

	/**
	 * @return string
	 */
	public static function get_automation_outcome_table_name() {
		global $wpdb;

		return $wpdb->prefix . 'shopmagic_automation_outcome';
	}

	/**
	 * @return string
	 */
	public static function get_outcome_logs_table_name() {
		global $wpdb;

		return $wpdb->prefix . 'shopmagic_automation_outcome_logs';
	}

	public function __construct( $base_file_name ) {
		$this->base_file_name = $base_file_name;
	}

	public function register_activation_hook() {
		register_activation_hook( $this->base_file_name, array( $this, 'install' ) );
	}

	/**
	 * @return bool
	 */
	public function is_old_database() {
		return $this->get_current_db_version() !== self::VERSION;
	}

	/**
	 * @return int
	 */
	private function get_current_db_version() {
		return (int) get_option( self::OPTION_NAME_WITH_VERSION, 0 );
	}

	/**
	 * @return bool
	 */
	private function v37() {
		global $wpdb;

		$table_name      = self::get_optin_email_table_name();
		$charset_collate = $wpdb->get_charset_collate();

		$table_optin_sql = "CREATE TABLE {$table_name} (
            id int(20) NOT NULL AUTO_INCREMENT,
			email varchar(255) NOT NULL,
			communication_type int(20) NOT NULL,
			created datetime NOT NULL,
			subscribe tinyint(1) NOT NULL,
			active tinyint(1) NOT NULL DEFAULT TRUE,
            PRIMARY KEY  (id)
        ) {$charset_collate};";

		$results = dbDelta( [ $table_optin_sql ] );

		return count( $results ) === 0;
	}

	/**
	 * @return bool
	 */
	private function v39() {
		global $wpdb;
		$charset_collate = $wpdb->get_charset_collate();
		$result          = true;

		$table_outcome_name = self::get_automation_outcome_table_name();
		$result             = $result && $wpdb->query( "DROP TABLE IF EXISTS {$table_outcome_name};" );

		$sql = "CREATE TABLE {$table_outcome_name} (
    		id int(20) NOT NULL AUTO_INCREMENT,
            execution_id varchar(48) NOT NULL,
			automation_id int(20) NOT NULL,
			automation_name varchar(255) NOT NULL,
			action_index varchar(255) NOT NULL,
			action_name varchar(255) NOT NULL,
			customer_id int(20),
			guest_id int(20),
			customer_email varchar(255) NOT NULL,
			success tinyint(1),
			finished tinyint(1) NOT NULL DEFAULT FALSE,
			created datetime NOT NULL,
			updated datetime NOT NULL,
            PRIMARY KEY  (id)
        ) {$charset_collate};";

		$result = $result && $wpdb->query( $sql );

		$table_name = self::get_outcome_logs_table_name();
		$result     = $result && $wpdb->query( "DROP TABLE IF EXISTS {$table_name};" );

		$sql = "CREATE TABLE {$table_name} (
    		id int(20) NOT NULL AUTO_INCREMENT,
    		execution_id varchar(48) NOT NULL,
			note varchar(2048) NOT NULL,
			created datetime NOT NULL,
            PRIMARY KEY  (id),
            KEY execution_id (execution_id)
        ) {$charset_collate};";

		$result = $result && $wpdb->query( $sql );

		return $result;
	}

	/**
	 *  Creates tables
	 */
	public function install() {
		global $wpdb;
		require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );

		$current_version = $this->get_current_db_version();
		$no_errors       = true;

		if ( $no_errors && $current_version < 37 ) {
			$no_errors = $no_errors && $this->v37();
		}
		if ( $no_errors && $current_version < 39 ) {
			$no_errors = $no_errors && $this->v39();
		}

		if ( $no_errors ) {
			add_action( 'wp_loaded', function () {
				if ( get_option( ListsOnAccount::ACCOUNT_PAGE_ID_OPTION_KEY ) === false ) {
					$this->add_account_communication_page();
				}

				update_option( self::OPTION_NAME_WITH_VERSION, self::VERSION );
			} );
		}
		if ( ! $no_errors ) {
			error_log( "Error while upgrading a database: " . $wpdb->last_error );
		}
	}

	private function add_account_communication_page() {
		$page_id = wp_insert_post( [
			'post_name'      => __( 'communication-preferences', 'shopmagic-for-woocommerce' ),
			'post_title'     => __( 'Communication preferences', 'shopmagic-for-woocommerce' ),
			'post_content'   => '[' . ListsOnAccount::ACCOUNT_SHORTCODE . ']',
			'post_status'    => 'publish',
			'post_type'      => 'page',
			'post_author'    => 1,
			'comment_status' => 'closed',
		] );
		update_option( ListsOnAccount::ACCOUNT_PAGE_ID_OPTION_KEY, $page_id );
	}
}
